/*
 * MILARGE.H -- LIBMI large object interface
 *
 *     Copyright c 1995 Illustra Information Technologies, Inc.
 *     All rights reserved.      See files/copyright.
 *
 */

#ifndef _MILARGE_H_
#define _MILARGE_H_

/*
 * legal flags for mi_large_object_open().
 */
#define MI_LO_RDONLY        	0x00000002
#define MI_LO_WRONLY        	0x00000004
#define MI_LO_RDWR          	0x00000008
#define MI_LO_TRUNC	  	0x00000010
#define MI_LO_APPEND	  	0x00000020

#define MI_LO_OPEN_FLAGS \
    (MI_LO_RDONLY|MI_LO_WRONLY|MI_LO_RDWR|MI_LO_TRUNC|MI_LO_APPEND)

/*
 * legal flags for mi_large_object_create and copy.
 */
#define MI_LO_EXTERNAL_FILE	0x80000000
#define MI_LO_INTERNAL_LO	0x00000000

/* legal storage managers for mi_large_object_create and copy */
#define MI_LO_NOSMGR		(-1)
#define MI_LO_DEFAULT_SMGR	0

/*
 * legal flags to the whence argument of
 * mi_large_object_seek, readwithseek and writewithseek
 */
#define MI_LO_SEEK_SET		0	
#define MI_LO_SEEK_CUR		1
#define MI_LO_SEEK_END		2

/* legal values for prot argument to mi_large_object_map */
#define MI_LO_PROT_READ		1
#define MI_LO_PROT_WRITE	2

/* legal values for flag argument to mi_large_object_map */
#define MI_LO_MAP_SHARED	1
#define MI_LO_MAP_PRIVATE	2

/* flag values in file transfer functions */
#define MI_LO_CLIENT_FILE	0x00000000
#define MI_LO_SERVER_FILE	0x00000001
#define MI_LO_SERVER_SLINK	0x00000003	/* sets SERVER_FILE bit, too */
#define MI_LO_NODUPS		0x00000004

/* temporary until user documentation get's fixed. */
#define MI_CLIENT_FILE		MI_LO_CLIENT_FILE
#define MI_SERVER_FILE		MI_LO_SERVER_FILE
#define MI_SERVER_SLINK		MI_LO_SERVER_SLINK

/* for filename spec */
#define MI_LO_WILDCARD		'?'
#define MI_LO_NOSUFFIX		'!'

typedef struct mi_lo_info
{
    mi_integer		minfo_len;
    mi_integer		minfo_smgr;
    mi_integer		minfo_asmgr;
    mi_oid		minfo_oid;
    mi_integer		minfo_cnt;
    mi_integer		minfo_lastmod;
    mi_integer		minfo_lastmodusec;
    mi_integer		minfo_archcnt;
    void MI_FAR		*minfo_dummy1;	/* linfo_storage */
#if !defined(ALPHA_OSF)
    void MI_FAR    	*minfo_dummy2;  /* XXX pad */
#endif /* !ALPHA_OSF */
    void MI_FAR		*minfo_dummy3;	/* linfo_astorage */
#if !defined(ALPHA_OSF)
    void MI_FAR    	*minfo_dummy4;  /* XXX pad */
#endif /* !ALPHA_OSF */
} MI_LOINFO;

/* struct returned by mi_large_object_stat */
typedef struct mi_lostat
{
    mi_integer	  	mist_len;
    MI_LOINFO     	mist_loinfo;
    mi_integer	  	mist_ino;
    mi_integer	  	mist_mode;
    mi_unsigned_integer mist_size;
    mi_unsigned_integer mist_sizehigh;
    mi_integer	  	mist_uid;
    mi_integer	  	mist_atime;
    mi_integer	  	mist_mtime;
    mi_integer 	  	mist_ctime;
    char	  	mist_type;
    char	  	mist_spare1;
    char	  	mist_spare2;
    char	  	mist_spare3;
} MI_LOSTAT;


typedef mi_integer	MI_LODESC;

typedef union
{
    void MI_FAR	*mr_data;
    MI_LOHANDLE	 mr_lo;
} MI_MULTIREP_DATA;

typedef unsigned char	MI_MULTIREP_SIZE;

/* defines for MI_MULTIREP_SIZE */
#define MI_MULTIREP_SMALL               0
#define MI_MULTIREP_LARGE               1

#define mi_issmall_data(size)	(!((size) & MI_MULTIREP_LARGE))
#define mi_set_large(size)	((void) ((size) |= MI_MULTIREP_LARGE))


EXTERNC_BEGIN

char MI_FAR * MI_PROC_EXPORT
mi_file_to_file ARGS((MI_CONNECTION MI_FAR *conn,
		      const char MI_FAR *fromfile,
		      mi_integer fromflags,
		      const char MI_FAR *tofile,
		      mi_integer toflags));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_file_to_large_object ARGS((MI_CONNECTION MI_FAR *conn,
			      const char MI_FAR *filename,
			      mi_integer flags,
			      mi_integer smgr,
			      mi_integer asmgr,
			      const char MI_FAR *smgropts,
			      const char MI_FAR *asmgropts));

mi_integer MI_PROC_EXPORT
mi_large_object_close ARGS((MI_CONNECTION MI_FAR *conn,
			    MI_LODESC lodesc));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_copy ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LOHANDLE MI_FAR *lo_handle,
			   const char MI_FAR *filename,
			   mi_integer flags,
			   mi_integer mode,
			   mi_integer smgr,
			   mi_integer asmgr));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_copy_opts ARGS((MI_CONNECTION MI_FAR *conn,
				MI_LOHANDLE MI_FAR *lo_handle,
				const char MI_FAR *filename,
				mi_integer flags,
				mi_integer mode,
				mi_integer smgr,
				mi_integer asmgr,
				const char MI_FAR *smgropts,
				const char MI_FAR *asmgropts));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_create ARGS((MI_CONNECTION MI_FAR *conn,
			     const char MI_FAR *filename,
			     mi_integer flags,
			     mi_integer mode,
			     mi_integer smgr,
			     mi_integer asmgr));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_create_opts ARGS((MI_CONNECTION MI_FAR *conn,
				  const char MI_FAR *filename,
				  mi_integer flags,
				  mi_integer mode,
				  mi_integer smgr,
				  mi_integer asmgr,
				  const char MI_FAR *smgropts,
				  const char MI_FAR *asmgropts));

mi_integer MI_PROC_EXPORT
mi_large_object_decrefcount ARGS((MI_CONNECTION MI_FAR *conn,
				  MI_LOHANDLE MI_FAR *lo_handle));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_expand ARGS((MI_CONNECTION MI_FAR *conn,
			     MI_MULTIREP_DATA MI_FAR *multidata,
			     mi_integer len,
			     mi_integer smgr,
			     mi_integer asmgr,
			     const char MI_FAR *smgropts,
			     const char MI_FAR *asmgropts));

char MI_FAR * MI_PROC_EXPORT
mi_large_object_filename ARGS((MI_CONNECTION MI_FAR *conn,
			       MI_LOHANDLE MI_FAR *lohandle,
			       const char MI_FAR *spec));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_gethandle ARGS((MI_CONNECTION MI_FAR *conn,
				const char MI_FAR *filename,
				mi_integer smgr));

mi_varlena MI_FAR * MI_PROC_EXPORT
mi_large_object_handles ARGS((MI_CONNECTION MI_FAR *conn,
			      MI_LOHANDLE MI_FAR * MI_FAR mi_lohandle[],
			      mi_integer cnt));

mi_integer MI_PROC_EXPORT
mi_large_object_increfcount ARGS((MI_CONNECTION MI_FAR *conn,
				  MI_LOHANDLE MI_FAR *lo_handle));

MI_LOINFO MI_FAR * MI_PROC_EXPORT
mi_large_object_info ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LOHANDLE MI_FAR *lo_handle));

char MI_FAR * MI_PROC_EXPORT
mi_large_object_map ARGS((MI_CONNECTION MI_FAR *conn,
			  MI_LODESC lodesc,
			  mi_integer len,
			  mi_integer prot,
			  mi_integer off,
			  mi_integer flags));

MI_LODESC MI_PROC_EXPORT
mi_large_object_open ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LOHANDLE MI_FAR *lo_handle,
			   mi_integer flags));

char MI_FAR * MI_PROC_EXPORT
mi_large_object_pin ARGS((MI_CONNECTION MI_FAR *conn,
			  mi_integer size,
			  MI_MULTIREP_DATA MI_FAR *multidata));

mi_integer MI_PROC_EXPORT
mi_large_object_read ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LODESC lodesc,
			   char MI_FAR *buf,
			   mi_integer nbytes));

mi_integer MI_PROC_EXPORT
mi_large_object_readwithseek ARGS((MI_CONNECTION MI_FAR *conn,
				   MI_LODESC lodesc,
				   char MI_FAR *buf,
				   mi_integer nbytes,
				   mi_integer off,
				   mi_integer whence));

mi_integer MI_PROC_EXPORT
mi_large_object_seek ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LODESC lodesc,
			   mi_integer offset,
			   mi_integer whence));

MI_LOSTAT MI_FAR * MI_PROC_EXPORT
mi_large_object_stat ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LODESC lodesc));

mi_integer MI_PROC_EXPORT
mi_large_object_tell ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_LODESC lodesc));

char MI_FAR * MI_PROC_EXPORT
mi_large_object_to_file ARGS((MI_CONNECTION MI_FAR *conn,
			      MI_LOHANDLE MI_FAR *lohandle,
			      const char MI_FAR *spec,
			      mi_integer flags));

MI_LOHANDLE MI_FAR * MI_PROC_EXPORT
mi_large_object_to_lohandle ARGS((mi_large_object MI_FAR *lo));

mi_integer MI_PROC_EXPORT
mi_large_object_unlink ARGS((MI_CONNECTION MI_FAR *conn,
			     MI_LODESC lodesc));

mi_integer MI_PROC_EXPORT
mi_large_object_unmap ARGS((MI_CONNECTION MI_FAR *conn,
			    MI_LODESC lodesc,
			    char MI_FAR *addr));

void MI_PROC_EXPORT
mi_large_object_unpin ARGS((MI_CONNECTION MI_FAR *conn,
			    mi_integer size,
			    MI_MULTIREP_DATA MI_FAR *multidata));

mi_integer MI_PROC_EXPORT
mi_large_object_write ARGS((MI_CONNECTION MI_FAR *conn,
			    MI_LODESC lodesc,
			    const char MI_FAR *buf,
			    mi_integer len));

mi_integer MI_PROC_EXPORT
mi_large_object_writewithseek ARGS((MI_CONNECTION MI_FAR *conn,
				    MI_LODESC lodesc,
				    const char MI_FAR *buf,
				    mi_integer len,
				    mi_integer off,
				    mi_integer whence));

mi_large_object MI_FAR * MI_PROC_EXPORT
mi_lohandle_to_large_object ARGS((MI_LOHANDLE MI_FAR *lohandle));


EXTERNC_END

#endif /* _MILARGE_H_ */
